<tbody>{% if rows %}{% for row in rows %}
    <tr>{% for cell in row %}
        <td>{% if cell.column.has_link %}<a href="{{ cell.link }}">{{ cell }}</a>{% else %}{{ cell }}{% endif %}</td>{% endfor %}
    </tr>{% endfor %}{% else %}
    <tr class="no-data">
        <td colspan="{{ num_columns }}">
            <p>
                Nothing to display.
            </p>
        </td>
    </tr>{% endif %}
</tbody>
